/********************************************************
 * PStack -- Demonstration on how to define a class	*
 *	that defines a private stack.			*
 *							*
 * Programming Problem: 14-3				*
 *							*
 * This program uses the stack class defined in 	*
 * Practical C++ Programming, Chapter 13.		*
 ********************************************************/
#include <stdlib.h>
#include <iostream.h>

const int STACK_SIZE = 100;	// Maximum size of a stack

/********************************************************
 * Stack class						*
 *							*
 * Member functions					*
 *	init -- initialize the stack.			*
 *	push -- put an item on the stack.		*
 *	pop -- remove an item from the stack.		*
 ********************************************************/
// The stack itself
class stack {
    private:
	int count;		// Number of items in the stack
	int data[STACK_SIZE];	// The items themselves
    public:
	// Initialize the stack
	void init(void);

	// Push an item on the stack
	void push(const int item);

	// Pop an item from the stack
	int pop(void);
};

/********************************************************
 * stack::init -- initialize the stack.			*
 ********************************************************/
inline void stack::init(void)
{
    count = 0;	// Zero the stack
}
/********************************************************
 * stack::push -- push an item on the stack.		*
 *							*
 * Warning: We do not check for overflow.		*
 *							*
 * Parameters						*
 *	item -- item to put in the stack		*
 ********************************************************/
inline void stack::push(const int item)
{
    data[count] = item;
    ++count;
}
/********************************************************
 * stack::pop -- get an item off the stack.		*
 *							*
 * Warning: We do not check for stack underflow.	*
 *							*
 * Returns						*
 * 	The top item from the stack.			*
 ********************************************************/
inline int stack::pop(void)
{
    // Stack goes down by one
    --count;

    // Then we return the top value
    return (data[count]);
}

/********************************************************
 * PStack -- Every instance of this class uses a 	*
 *	common stack.					*
 *							*
 * Member functions:					*
 *	lock -- locks the stack for exclusive access	*
 *	unlock -- unlocks the stack.			*
 *	test_lock -- tests to see if the lock is in 	*
 *			place.				*
 ********************************************************/
class pstack {
    private:
	static stack my_stack;	// Stack we are using
	static int serial;	// Serial number for the pstack
	static int locker;	// Who has the stack locked now

	int my_serial;		// My serial number
    
    public:
	pstack(void) {
	    // Record my serial number
	    my_serial = serial;
	    ++serial;
	}
	// Destructor defaults

    private:
	pstack(const pstack &);	// No copy constructor
	pstack operator =(const pstack &);// No assignment operator
    
    public:
	// Lock the stack 
	void lock(void) {
	    locker = my_serial;
	}

	// Unlock the stack
	void unlock(void) {
	    if (my_serial != locker) {
		cerr << "Error: Illegal unlock attempt\n";
	    } else
		locker = 0;
	}

	static int test_lock(void) {
	    return (locker != 0);
	}
};

int pstack::serial = 1;		// Serial number
int pstack::locker = 0;		// Who's got the stack locked

main()
{
    pstack a_stack;	// Stack we want to use
   
    // Testing is left to the student
    return (0);
}
