/********************************************************
 * conv -- convert a file containing ASCII numbers to	*
 *	binary numbers or binary to ASCII		*
 *							*
 * Programming Problem: 16-4.				*
 *							*
 * Usage:						*
 *	conv <how> <in_file> <out_file>			*
 *							*
 *	How:						*
 *		-b -- convert ascii to binary		*
 *		-a -- convert binary to ascii		*
 ********************************************************/
#include <iostream.h>
#include <fstream.h>

/********************************************************
 * bin_2_ascii -- convert a file from binary to ascii	*
 ********************************************************/
void bin_2_ascii(
    const char in_name[],	// Name of the input file
    const char out_name[]	// Name of the output file
) {
    // Input file
    ifstream in_file(in_name, ios::bin|ios::in);

    ofstream out_file(out_name);	// Output file

    if (in_file.bad()) {
	cerr << "Error: Could not open " << in_name << " for input\n";
	exit (8);
    }

    if (out_file.bad()) {
	cerr << "Error: Could not open " << out_name << " for output\n";
	exit (8);
    }

    while (!in_file.eof()) {
	int number;		// Number we are moving

	// Do a read read to get the number
	in_file.read(&number, sizeof(number));
	if (in_file.eof())
	    break;

	out_file << number << '\n';	// Dump the number
    }
}

/********************************************************
 * ascii_2_bin -- convert a file from ascii to binary	*
 ********************************************************/
void ascii_2_bin(
    const char in_name[],	// Name of the input file
    const char out_name[]	// Name of the output file
) {
    // Input file
    ifstream in_file(in_name);

    ofstream out_file(out_name, ios::bin|ios::out);	// Output file

    if (in_file.bad()) {
	cerr << "Error: Could not open " << in_name << " for input\n";
	exit (8);
    }

    if (out_file.bad()) {
	cerr << "Error: Could not open " << out_name << " for output\n";
	exit (8);
    }

    while (!in_file.eof()) {
	int number;		// Number we are moving

	// Do a read read to get the number
	in_file >> number;
	if (in_file.eof())
	    break;

	out_file.write(&number, sizeof(number));	// Dump the number
    }
}

/********************************************************
 * Usage -- tell the user what to do with us		*
 ********************************************************/
void usage(void)
{
   cerr << "Usage is: conv <-b|-a> <in_file> <out_file>\n";
   exit (8);
}

main(int argc, char *argv[])
{
    if (argc != 4) 
	usage();

    if (strcmp(argv[1], "-a") == 0) {
	bin_2_ascii(argv[2], argv[3]);
    } else if (strcmp(argv[1], "-b") == 0) {
	ascii_2_bin(argv[2], argv[3]);
    } else
	usage();
    
    return (0);
}
    
