/********************************************************
 * date -- date class definition and tests		*
 *							*
 * Programming Problem: 18-5				*
 ********************************************************/
#include <iomanip.h>
#include <iostream.h>

/********************************************************
 * date -- a class to handle dates in the form mm/dd	*
 *		mm -- month				*
 *		dd -- date				*
 *							*
 * Member functions					*
 *	date	-- Create date of (00:00:00)		*
 *	date(mm, dd) -- create date from 2 numbers	*
 *	set(mm, dd) -- Set the date			*
 *	get_mm()	-- Get month			*
 *	get_dd()	-- Get day			*
 *	t = t + t	-- Add date			*
 *	t = t - t	-- Add date			*
 *	t += t;		-- Add one date to another	*
 *	t -= t;		-- Sub one date from another	*
 *	t == t		-- Compare date			*
 *	t != t		-- Compare date			*
 *	t <= t		-- Compare date			*
 *	t >= t		-- Compare date			*
 *	t < t		-- Compare date			*
 *	t > t		-- Compare date			*
 *	out_file << date  -- Write date			*
 *	in_file >> date -- read date			*
 *	julian		-- Julian data			*
 *	set_julian(jd)	-- Set date (julian form)	*
 ********************************************************/
// Number of days per month
static unsigned short int days_per_month[] = {
    31,		/* Jan */
    28,		/* Feb */
    31,		/* Mar */
    30,		/* April */
    31,		/* May */
    30,		/* June */
    31,		/* July */
    31,		/* August */
    30,		/* Sept. */
    31,		/* Oct */
    30,		/* Nov */
    31		/* Dec */
};
// Julian days (days since Jan 1) for the first of each month 
static unsigned short int julian_days_per_month[12] = { 0 };
class date {
    private:
        short int mm, dd;	// Month and day
	static void init_julian(void);
    public:
        date(void) { 
	    init_julian();
	    mm = dd = 1;
	}
	date( 
	    const short int i_mm,
	    const short int i_dd
	) {
	    init_julian();
	    set(i_mm, i_dd);
	}
	// Destructor defaults
	// Copy constructor defaults
	// Assignment operator defaults
	void set( 
	    const short int i_mm,
	    const short int i_dd
	) {
	    mm = i_mm;
	    dd = i_dd;
	}

	const short int get_mm(void) const { return (mm);}
	const short int get_dd(void) const { return (dd);}

	date operator +(const date &oper2) const {
	    date result(*this);	// Result of the add
	    result += oper2;
	    return (result);
	}
	date operator -(const date &oper2) const {
	    date result(*this);	// Result of the subtract
	    result -= oper2;
	    return (result);
	}

	date &operator +=(const date &oper2) {
	    set_julian(julian() + oper2.julian());
	    return (*this);
	}
	date &operator -=(const date &oper2) {
	    set_julian(julian() - oper2.julian());
	    return (*this);
	}

	int operator ==(const date &oper2) const {
	    return(julian() == oper2.julian());
	}
	int operator != (const date &oper2) const {
	    return(julian() != oper2.julian());
	}
	int operator < (const date &oper2) const {
	    return(julian() < oper2.julian());
	}
	int operator <= (const date &oper2) const {
	    return(julian() <= oper2.julian());
	}
	int operator > (const date &oper2) const {
	    return(julian() > oper2.julian());
	}
	int operator >= (const date &oper2) const {
	    return(julian() >= oper2.julian());
	}
	void set_julian(long int julian);
	const long int julian(void) const {
	    return (julian_days_per_month[mm-1] + dd);
	}

    friend ostream &operator <<(ostream &out_file, const date oper2);
    friend istream &operator >>(istream &out_file, const date oper2);
};
/********************************************************
 * set_julian -- set a day base on a julian date	*
 ********************************************************/
void date::set_julian(long int julian) 
{
    while (julian < 1)
	julian += 365;

    while (julian > 365)
        julian -= 365;

    int i;	// Index into month table

    for (i = 0; i < 12; ++i) {
        if (julian_days_per_month[i+1] > julian) {
	    dd = (unsigned short int)(julian - julian_days_per_month[i]);
	    mm = i+1;
	    return;
	}
    }
    dd = (unsigned short int)(julian - julian_days_per_month[11 -1]);
    mm = 12;
}
    
/********************************************************
 * init_julian -- initialize the julian date table	*
 ********************************************************/
void date::init_julian(void) 
{
    if (julian_days_per_month[1] != 0)
        return;

    int i;	// Index into the months

    for (i = 1; i < 12; i++) {
        julian_days_per_month[i] = julian_days_per_month[i-1] +
				   days_per_month[i-1];
    }
}
/********************************************************
 * << date -- output a date on a given file		*
 ********************************************************/
ostream &operator <<(
    ostream &out_file,
    const date oper2
) {
    out_file << setw(2) <<  oper2.mm << '/';
    out_file << setw(2) <<  oper2.dd;
    return (out_file);
}

main()
{
    date d1(1,2);
    date d2(7,7);

    cout << "d1 = " << d1 << '\n';
    cout << "d2 = " << d2 << '\n';
    cout << "d1 + d2 = " << (d1 + d2) << '\n';
    cout << "d1 - d2 = " << (d1 - d2) << '\n';
    cout << "d2 - d1 = " << (d2 - d1) << '\n';
    cout << "d1 == d2 = " << (d1 == d2) << '\n';
    cout << "d1 != d2 = " << (d1 != d2) << '\n';
    cout << "d1 <= d2 = " << (d1 <= d2) << '\n';
    cout << "d1 >= d2 = " << (d1 >= d2) << '\n';
    return (0);
}

