/********************************************************
 * time -- time class definition and tests		*
 *							*
 * Programming Problem: 18-4				*
 ********************************************************/
#include <iomanip.h>
#include <iostream.h>

/********************************************************
 * time -- a class to handle times in the form hh:mm:ss	*
 *		hh -- hours (0-23)			*
 *		mm -- minutes (0-59)			*
 *		ss -- seconds (0-59)			*
 *							*
 * Member functions					*
 *	time	-- Create time of (00:00:00)		*
 *	time(hh, mm, ss=0) -- create time from 3 	*
 *			numbers.			*
 *	set(hh, mm, ss=0) -- Set the time		*
 *	get_hh()	-- Get hours			*
 *	get_mm()	-- Get minutes			*
 *	get_ss()	-- Get seconds			*
 *	seconds()	-- Get time as seconds since	*
 *			midnight			*
 *	t = t + t	-- Add time			*
 *	t = t - t	-- Add time			*
 *	t += t;		-- Add one time to another	*
 *	t -= t;		-- Sub one time from another	*
 *	t == t		-- Compare times		*
 *	t != t		-- Compare times		*
 *	t <= t		-- Compare times		*
 *	t >= t		-- Compare times		*
 *	t < t		-- Compare times		*
 *	t > t		-- Compare times		*
 *	out_file << time  -- Write time			*
 *	in_file >> time -- read time			*
 ********************************************************/
class time {
    private:
        short int hh, mm, ss;	// Hours minutes seconds
    public:
        time(void) { hh = mm = ss = 0;}
	time( 
	    const short int i_hh,
	    const short int i_mm,
	    const short int i_ss = 0
	) {
	    set(i_hh, i_mm, i_ss);
	}
	// Destructor defaults
	// Copy constructor defaults
	// Assignment operator defaults
	void set( 
	    const short int i_hh,
	    const short int i_mm,
	    const short int i_ss = 0
	) {
	    hh = i_hh;
	    mm = i_mm;
	    ss = i_ss;
	}

	const short int get_hh(void) const { return (hh);}
	const short int get_mm(void) const { return (mm);}
	const short int get_ss(void) const { return (ss);}

	const unsigned long int seconds(void) const {
	    return ((hh * 60 * 60) + (mm * 60) + ss);
	}
	time operator +(const time &oper2) const {
	    time result(*this);	// Result of the add
	    result += oper2;
	    return (result);
	}
	time operator -(const time &oper2) const {
	    time result(*this);	// Result of the subtract
	    result -= oper2;
	    return (result);
	}

	time &operator +=(const time &oper2) {
	    hh += oper2.hh;
	    mm += oper2.mm;
	    ss += oper2.ss;
	    if (ss >= 60) {
	        mm += ss / 60;
		ss %= 60;
	    }
	    if (mm >= 60) {
	        hh += mm / 60;
		mm %= 60;
	    }
	    hh %= 24;
	    return (*this);
	}
	time &operator -=(const time &oper2) {
	    hh -= oper2.hh;
	    mm -= oper2.mm;
	    ss -= oper2.ss;
	    if (ss < 0) {
		--mm;
		ss += 60;
	    }
	    if (mm < 0) {
	        --hh;
		mm += 60;
	    }
	    if (hh < 0)
	        hh += 24;
	    return (*this);
	}

	int operator ==(const time &oper2) const {
	    return ((hh == oper2.hh) && 
	            (mm == oper2.mm) && 
		    (ss == oper2.ss));
	}
	int operator != (const time &oper2) const {
	    return (! (*this == oper2));
	}
	int operator < (const time &oper2) const {
	    if (hh < oper2.hh) return (1);
	    if (hh > oper2.hh) return (0);

	    if (mm < oper2.mm) return (1);
	    if (mm > oper2.mm) return (0);

	    if (ss < oper2.ss) return (1);
	    if (ss > oper2.ss) return (0);
	    // Equal
	    return (0);
	}
	int operator <= (const time &oper2) const {
	    if (hh < oper2.hh) return (1);
	    if (hh > oper2.hh) return (0);

	    if (mm < oper2.mm) return (1);
	    if (mm > oper2.mm) return (0);

	    if (ss < oper2.ss) return (1);
	    if (ss > oper2.ss) return (0);
	    // Equal
	    return (1);
	}
	int operator > (const time &oper2) const {
	    if (hh > oper2.hh) return (1);
	    if (hh < oper2.hh) return (0);

	    if (mm > oper2.mm) return (1);
	    if (mm < oper2.mm) return (0);

	    if (ss > oper2.ss) return (1);
	    if (ss < oper2.ss) return (0);
	    // Equal
	    return (0);
	}
	int operator >= (const time &oper2) const {
	    if (hh > oper2.hh) return (1);
	    if (hh < oper2.hh) return (0);

	    if (mm > oper2.mm) return (1);
	    if (mm < oper2.mm) return (0);

	    if (ss > oper2.ss) return (1);
	    if (ss < oper2.ss) return (0);
	    // Equal
	    return (1);
	}

    friend ostream &operator <<(ostream &out_file, const time oper2);
    friend istream &operator >>(istream &out_file, const time oper2);
};
/********************************************************
 * << time -- output a time on a given file		*
 ********************************************************/
ostream &operator <<(
    ostream &out_file,
    const time oper2
) {
    out_file << setw(2) << setfill('0') << oper2.hh << ':';
    out_file << setw(2) << setfill('0') << oper2.mm << ':';
    out_file << setw(2) << setfill('0') << oper2.ss;
    return (out_file);
}

main()
{
    time t1(1,2,3);
    time t2(1,2,4);

    cout << "t1 = " << t1 << '\n';
    cout << "t2 = " << t2 << '\n';
    cout << "t1 + t2 = " << (t1 + t2) << '\n';
    cout << "t1 - t2 = " << (t1 - t2) << '\n';
    cout << "t2 - t1 = " << (t2 - t1) << '\n';
    cout << "t1 == t2 = " << (t1 == t2) << '\n';
    cout << "t1 != t2 = " << (t1 != t2) << '\n';
    cout << "t1 <= t2 = " << (t1 <= t2) << '\n';
    cout << "t1 >= t2 = " << (t1 >= t2) << '\n';
    return (0);
}

