/********************************************************
 * tree -- Tree with delete function tester		*
 *							*
 * Programming Problem: 20-4				*
 *							*
 * Based on the Words program from Practical C++	*
 *                                                      *
 * Usage:                                               *
 *      tree                                     	*
 ********************************************************/
#include <iostream.h>
#include <fstream.h>
#include <iomanip.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>

class tree {
    private:
	// The basic node of a tree
	class node {
	    private:
		node    *right;      // tree to the right
		node    *left;       // tree to the left
	    public:
		char    *data; 	     // data for this tree

	    friend class tree;	    
	};

	// the top of the tree 
	node *root;

	// Enter a new node into a tree or sub-tree
	void enter_one(
	   node *&node, 		// Node of sub-tree to look at
	   const char *const data	// Word to add
	);

	// Print a single node
	void print_one(node *top);

	// Remove a node from a tree
	void remove_one(node *&sub_tree, const char *const data);

    public:
	tree(void) { root = NULL;}

	// Add a new data to our tree
        void enter(const char *const data) {
	    enter_one(root, data);
	}

	// Print the tree
	void print(void) {
	    print_one(root);
	}

	// Remove a node from a tree
	void remove(const char *const data);
};


/********************************************************
 * tree::enter_one -- enter a data into the tree        *
 *                                                      *
 * Parameters                                           *
 *      new_node -- current node we are looking at      *
 *      data -- data to enter                           *
 ********************************************************/
void tree::enter_one(
   node *&new_node, 		// Sub-tree to look at
   const char *const data	// Word to add
)
{
    int  result;        // result of strcmp

    // see if we have reached the end
    if (new_node == NULL) {
	new_node = new node;

	new_node->left = NULL;
	new_node->right = NULL;
	new_node->data = strdup(data);
    }
    result = strcmp(new_node->data, data);
    if (result == 0) {
        return;
    }

    if (result < 0)
        enter_one(new_node->right, data);
    else
        enter_one(new_node->left, data);
}

/********************************************************
 * tree::print_one -- print out the words in a tree    	*
 *                                                      *
 * Parameters                                           *
 *      top -- the root of the tree to print            *
 ********************************************************/
void tree::print_one(node *top)
{
    if (top == NULL)
        return;                 // short tree 

    print_one(top->left);
    cout << top->data << '\n';
    print_one(top->right);
}

/********************************************************
 * tree::remove -- remove a node from the tree		*
 ********************************************************/
void tree::remove(
    const char *const data	// Word to remove
)
{
    remove_one(root, data);
}
/********************************************************
 * remove_one -- remove a node from the tree		*
 ********************************************************/
void tree::remove_one(
    node *&sub_tree, 		// Root of the sub-tree
    const char *const data	// Data we are looking for
) {
    int  result;        // result of strcmp

    // see if we have reached the end
    if (sub_tree == NULL)
	return;		// Could not find item to delete

    result = strcmp(sub_tree->data, data);
    if (result == 0) {
	delete []sub_tree->data;	// Remove the data
	node *old_sub_tree = sub_tree;	// Save pointer to old tree

	if (sub_tree->right == NULL) {
	    sub_tree = sub_tree->left;
	    delete old_sub_tree;
	    return;
	}
	if (sub_tree->left == NULL) {
	    sub_tree = sub_tree->right;
	    delete old_sub_tree;
	    return;
	}
	// Move all the data in the right sub tree up
	while (sub_tree->right != NULL) {
	    sub_tree->data = sub_tree->right->data;
	    sub_tree = sub_tree->right;
	}
	delete sub_tree;	// Remove the last right node
    }

    if (result < 0)
	remove_one(sub_tree->right, data);
    else
	remove_one(sub_tree->left, data);
}

main()
{
    tree test_tree;	// A tree to test

    test_tree.enter("Now");
    test_tree.enter("Is");
    test_tree.enter("The");
    test_tree.enter("Time");
    test_tree.enter("Able");
    test_tree.enter("Bake");
    test_tree.enter("Sam");

    test_tree.print();

    cout << "------------------ delete Time\n";
    test_tree.remove("Time");
    test_tree.print();

    return (0);
}
