/********************************************************
 * check -- checkbook class and test routine		*
 *							*
 * Programming Problem: 21-1.				*
 ********************************************************/
#include <iostream.h>
/********************************************************
 * Checkbook class.   Keeps a running total of what's	*
 *	in your checkbook.				*
 *							*
 * Note: We store money as long integer pennies		*
 *							*
 * Member functions:					*
 *	add_item -- Put another item on the books	*
 *	total	-- Return the total of the items put in	*
 ********************************************************/
class checkbook {
    private:
	long int balance;	// Current balance (in pennies)
    
    public:
	checkbook(void) {
	    balance = 0;
	}
	// Destructor defaults
	// Copy constructor defaults
	// Assignment operator defaults

	// Add an item to the checkbook
	void add_item(long int amount) {
	    balance += amount;
	}

	// Return the running total
	long int total(void) {
	    return (balance);
	}
};

/********************************************************
 * Queue class.						*
 *	A queue or FIFO is a first in, first out data	*
 *	storage.					*
 * 							*
 *	Warning: We do not check for overfilling the 	*
 *	queue or emptying too much data.		*
 *							*
 * Member functions:					*
 *	put  -- puts an item in the queue		*
 *	print -- prints the entries in the queue	*
 ********************************************************/
const int QUEUE_SIZE = 5;	// Max number of elements in queue

class queue {
    private:
	unsigned short int put_index;	// Index for the next get
	unsigned int n_items;		// Number of items in the queue
	long int data[QUEUE_SIZE];		// Data in the queue

	// Increments (and wraps) a queue index
	void inc_index(
	    unsigned short int &index		// Index to increment
	) {
	    ++index;		// Move up
	    index %= QUEUE_SIZE;// Wrap if needed
	}
    public:
	queue(void) {
	   put_index = 0;
	   n_items = 0;
	}
	// Destructor defaults
    private:
	queue(const queue &);	// No copy constructor

	// No assignment operator
	queue operator= (const queue &);
    public:
	// Adds an item to the queue
	void put(
	    long int item		// Item to put on the queue
	) {
	    data[put_index] = item;
	    inc_index(put_index);
	    ++n_items;
	}
	void print_entries(void);
};
/********************************************************
 * queue::print_entries -- print the entries in the 	*
 *			queue.				*
 ********************************************************/
void queue::print_entries(void) {
    unsigned short int item_index;	// Index into the items
    int i;		// Item counter
    int limit;		// How many items to print


    if (n_items < QUEUE_SIZE) {
	item_index = 0;
	limit = n_items;
    } else {
	item_index = put_index;
	limit = QUEUE_SIZE;
    }

    for (i = 0; i < limit; ++i) {
	cout << data[item_index] << '\n';
	inc_index(item_index);
    }
}

/********************************************************
 * record_checkbook -- recording checkbook class	*
 *							*
 * Member functions:					*
 *	add_item -- put an item in the checkbook	*
 *	print_entries -- print the last few entries	*
 *		in the checkbook.			*
 *	total -- return the total amount in the account	*
 *							*
 * Note: print_entries only prints amounts.		*
 *	All amounts are in pennies			*
 *							*
 * Warning: This class does not demonstrate derivation	*
 *	as the teacher intended.  Instead it 		*
 *	demonstrates encapsulation.			*
 ********************************************************/
class record_checkbook {
    private:
	checkbook my_checkbook;
	queue	  my_queue;
    public:
	void add_item(const long int amount) {
	    my_checkbook.add_item(amount);
	    my_queue.put(amount);
	}
	void print_entries(void) {
	    my_queue.print_entries();
	}
	int total() {
	    return (my_checkbook.total());
	}
};
    


main()
{
    record_checkbook my_checkbook;	// Checkbook to test

    while (1) {
	long int cur_check;	// Check we just wrote (<0)
				// or deposit we just made (>0)

	cout << "Enter amount (in pennies) : ";
	cin >> cur_check;

	my_checkbook.add_item(cur_check);
	my_checkbook.print_entries();
	cout << "Current total: " << my_checkbook.total() << '\n';

	char answer;	// More? answer

	cout << "More [y/n] ";
	cin >> answer;

	if ((answer == 'n') || (answer == 'N'))
	    break;
    }
    return (0);
}
